package com.game.business.service;

import java.util.List;
import com.game.business.domain.AppUsersCashrecord;
import com.baomidou.mybatisplus.extension.service.IService;

/**
 * 提现记录Service接口
 *
 * @author game
 * @date 2024-06-18
 */
public interface IAppUsersCashrecordService extends IService<AppUsersCashrecord> {
    /**
     * 查询提现记录
     *
     * @param id 提现记录主键
     * @return 提现记录
     */
    public AppUsersCashrecord selectAppUsersCashrecordById(Long id);

    /**
     * 查询提现记录列表
     *
     * @param appUsersCashrecord 提现记录
     * @return 提现记录集合
     */
    public List<AppUsersCashrecord> selectAppUsersCashrecordList(AppUsersCashrecord appUsersCashrecord);

    /**
     * 新增提现记录
     *
     * @param appUsersCashrecord 提现记录
     * @return 结果
     */
    public int insertAppUsersCashrecord(AppUsersCashrecord appUsersCashrecord);

    /**
     * 修改提现记录
     *
     * @param appUsersCashrecord 提现记录
     * @return 结果
     */
    public int updateAppUsersCashrecord(AppUsersCashrecord appUsersCashrecord);

    /**
     * 批量删除提现记录
     *
     * @param ids 需要删除的提现记录主键集合
     * @return 结果
     */
    public int deleteAppUsersCashrecordByIds(Long[] ids);

    /**
     * 删除提现记录信息
     *
     * @param id 提现记录主键
     * @return 结果
     */
    public int deleteAppUsersCashrecordById(Long id);

    /**
     *  根据日期查询用户提现记录
     *
     * */
    public List<AppUsersCashrecord> getUserCashList(Long userId,String beginTime,String endTime);

    /**
     *  根据日期查询用户提现记录
     *
     * */
    public List<AppUsersCashrecord> getUserCashListByIds(List<Long> userIds,String beginTime,String endTime);

    public Double sumWithdraw(Long userId);

    public AppUsersCashrecord getByOrderNo(String orderNo);
}